/***************************************************************************
 *   Copyright (C) 2006 by Borko Boskovic                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

void boardGenerateNonQuiescentMoves(const Board* board, Moves* moves, gint historyHeuristicsMatrix[64][64]){
	guint64 pieces, destination, freeSquares;
	guint from, to, player, opponent;

	player = board->player;
	opponent = !player;
	freeSquares = board->freeSquares;
	moves->size = 0;
	
	/* Pawn moves */
	if(player == WHITE){
		pieces = board->pawns[WHITE];
		/* One square up */
		destination = shiftUp(pieces) & freeSquares & ~RANK8;
		while(destination){
			to = getFirstBitAndClear64(&destination);
			from = to - 8;
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = historyHeuristicsMatrix[from][to];
			moves->size ++;
		}
		/* Two square up */
		destination = shiftUp(shiftUp(pieces) & freeSquares) & RANK4 & freeSquares;
		while(destination){
			to = getFirstBitAndClear64(&destination);
			from = to - 16;
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = historyHeuristicsMatrix[from][to];
			moves->size ++;
		}
		/* Castling */
		if(board->castling & WHITE_CASTLING){
			if(board->castling & WHITE_CASTLING_KING_SIDE && ((F1G1&freeSquares) == F1G1)){
				moves->move[moves->size] = moveGenerate(e1,g1);
				moves->value[moves->size] = historyHeuristicsMatrix[e1][g1];
				moves->size ++;
			}
			if(board->castling & WHITE_CASTLING_QUEEN_SIDE && ((B1C1D1&freeSquares) == B1C1D1)){
				moves->move[moves->size] = moveGenerate(e1,c1);
				moves->value[moves->size] = historyHeuristicsMatrix[e1][c1];
				moves->size ++;
			}
		}
	}else{
		pieces = board->pawns[BLACK];
		/* One square down */
		destination = shiftDown(pieces) & freeSquares & ~RANK1;
		while(destination){
			to = getFirstBitAndClear64(&destination);
			from = to + 8;
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = historyHeuristicsMatrix[from][to];
			moves->size ++;
		}
		/* Two square down */
		destination = shiftDown(shiftDown(pieces) & freeSquares ) & RANK5 & freeSquares;
		while(destination){
			to = getFirstBitAndClear64(&destination);
			from = to + 16;
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = historyHeuristicsMatrix[from][to];
			moves->size ++;
		}
		/* Castling */
		if(board->castling & BLACK_CASTLING){
			if(board->castling & BLACK_CASTLING_KING_SIDE && ((F8G8&freeSquares) == F8G8)){
				moves->move[moves->size] = moveGenerate(e8,g8);
				moves->value[moves->size] = historyHeuristicsMatrix[e8][g8];
				moves->size ++;
			}
			if(board->castling & BLACK_CASTLING_QUEEN_SIDE && ((B8C8D8&freeSquares) == B8C8D8)){
				moves->move[moves->size] = moveGenerate(e8,c8);
				moves->value[moves->size] = historyHeuristicsMatrix[e8][c8];
				moves->size ++;
			}
		}

	}
	
	/* Knights moves */
	pieces = board->knights[player];
	while (pieces) {
		from = getFirstBitAndClear64(&pieces);
		destination = attackKnight[from] & freeSquares;
		while(destination) {
			to = getFirstBitAndClear64(&destination);
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = historyHeuristicsMatrix[from][to];
			moves->size ++;
		}
	}
	
	/* Bishop moves */
	pieces = board->bishops[player];
	while (pieces) {
		from = getFirstBitAndClear64(&pieces);
		destination = attackBishop(board,from) & freeSquares;
		while(destination) {
			to = getFirstBitAndClear64(&destination);
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = historyHeuristicsMatrix[from][to];
			moves->size ++;
		}
	}
	
	/* Rook moves */
	pieces = board->rooks[player];
	while (pieces) {
		from = getFirstBitAndClear64(&pieces);
		destination = attackRook(board,from) & freeSquares;
		while(destination) {
			to = getFirstBitAndClear64(&destination);
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = historyHeuristicsMatrix[from][to];
			moves->size ++;
		}
	}
	
	/* Queen moves */
	pieces = board->queens[player];
	while (pieces) {
		from = getFirstBitAndClear64(&pieces);
		destination = attackQueen(board,from) & freeSquares;
		while(destination) {
			to = getFirstBitAndClear64(&destination);
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = historyHeuristicsMatrix[from][to];
			moves->size ++;
		}
	}

	/* King moves */
	from = indexOfFirstBit64(&board->king[player]);		
	destination = attackKing[from] & freeSquares;
	while(destination) {
		to = getFirstBitAndClear64(&destination);
		moves->move[moves->size] = moveGenerate(from,to);
		moves->value[moves->size] = historyHeuristicsMatrix[from][to];
		moves->size ++;
	}
	moves->next = moves->sorted = 0;
}

void boardGenerateQuiescentMoves(const Board* board, Moves* moves){
	guint64 pieces, destination, attack, freeSquares;
	guint from, to, pPiece,player,opponent;

	player = board->player;
	opponent = !player;
	freeSquares = board->freeSquares;
	moves->size = 0;
	
	/* Pawn moves */
	if(player == WHITE){
		pieces = board->pawns[WHITE];
		/* One square up */
		destination = shiftUp(pieces) & freeSquares & RANK8;
		while(destination){
			to = getFirstBitAndClear64(&destination);
			from = to - 8;
			for(pPiece = WHITE_QUEEN; pPiece > WHITE_PAWN; pPiece-=2){
					moves->move[moves->size] = moveGeneratePromotion(from,to,pPiece);
					moves->value[moves->size] = pieceValue[pPiece];
					moves->size ++;
			}
		}
		/* Right capture */
		destination = shiftUpRight(pieces) & board->pieces[BLACK];
		while(destination){
			to = getFirstBitAndClear64(&destination);
			from = to - 9;
			if(to >= a8){
				for(pPiece = WHITE_QUEEN; pPiece > WHITE_PAWN; pPiece-=2){
					moves->move[moves->size] = moveGeneratePromotion(from,to,pPiece);
					moves->value[moves->size] = pieceValue[pPiece] + pieceValue[board->types[to]];
					moves->size ++;
				}
			}
			else{
				moves->move[moves->size] = moveGenerate(from,to);
				moves->value[moves->size] = pieceValue[board->types[to]] - PAWN_VALUE;
				moves->size ++;
			}
		}
		/* Left capture */
		destination = shiftUpLeft(pieces) & board->pieces[BLACK];
		while(destination){
			to = getFirstBitAndClear64(&destination);
			from = to - 7;
			if(to >= a8){
				for(pPiece = WHITE_QUEEN; pPiece > WHITE_PAWN; pPiece-=2){
					moves->move[moves->size] = moveGeneratePromotion(from,to,pPiece);
					moves->value[moves->size] = pieceValue[pPiece] + pieceValue[board->types[to]];
					moves->size ++;
				}
			}
			else{
				moves->move[moves->size] = moveGenerate(from,to);
				moves->value[moves->size] = pieceValue[board->types[to]] - PAWN_VALUE;
				moves->size ++;
			}
		}
		/* En passant */
		if(board->enPassant){
			destination = pieces & attackPawn[BLACK][board->enPassant];
			while(destination){
				from = getFirstBitAndClear64(&destination);
				moves->move[moves->size] = moveGenerate(from,board->enPassant);
				moves->value[moves->size] = 0;
				moves->size ++;
			}
		}
	}else{
		pieces = board->pawns[BLACK];
		/* One square down */
		destination = shiftDown(pieces) & freeSquares & RANK1;
		while(destination){
			to = getFirstBitAndClear64(&destination);
			from = to + 8;
			for(pPiece = BLACK_QUEEN; pPiece > BLACK_PAWN; pPiece-=2){
				moves->move[moves->size] = moveGeneratePromotion(from,to,pPiece);
				moves->value[moves->size] = pieceValue[pPiece];
				moves->size ++;
			}
		}
		/* Right capture  (pohitritev -> legalDestination)*/
		destination = shiftDownRight(pieces) & board->pieces[WHITE];
		while(destination){
			to = getFirstBitAndClear64(&destination);
			from = to + 7;
			if(to <= h1){
				for(pPiece = BLACK_QUEEN; pPiece > BLACK_PAWN; pPiece-=2){
					moves->move[moves->size] = moveGeneratePromotion(from,to,pPiece);
					moves->value[moves->size] = pieceValue[pPiece] + pieceValue[board->types[to]];
					moves->size ++;
				}
			}
			else{
				moves->move[moves->size] = moveGenerate(from,to);
				moves->value[moves->size] = pieceValue[board->types[to]] - PAWN_VALUE;
				moves->size ++;
			}
		}
		/* Left capture */
		destination = shiftDownLeft(pieces) & board->pieces[WHITE];
		while(destination){
			to = getFirstBitAndClear64(&destination);
			from = to + 9;
			if(to <= h1){
				for(pPiece = BLACK_QUEEN; pPiece > BLACK_PAWN; pPiece-=2){
					moves->move[moves->size] = moveGeneratePromotion(from,to,pPiece);
					moves->value[moves->size] = pieceValue[pPiece] + pieceValue[board->types[to]];
					moves->size ++;
				}
			}
			else{
				moves->move[moves->size] = moveGenerate(from,to);
				moves->value[moves->size] = pieceValue[board->types[to]] - PAWN_VALUE;
				moves->size ++;
			}
		}
		/* En passant */
		if(board->enPassant){
			destination = pieces & attackPawn[WHITE][board->enPassant];
			while(destination){
				from = getFirstBitAndClear64(&destination);
				moves->move[moves->size] = moveGenerate(from,board->enPassant);
				moves->value[moves->size] = 0;
				moves->size ++;
			}
		}
	}
	/* Knights moves */
	pieces = board->knights[player];
	while (pieces) {
		from = getFirstBitAndClear64(&pieces);
		attack = attackKnight[from];
		destination = attack & board->pieces[opponent]; /* Captures */
		while(destination) {
			to = getFirstBitAndClear64(&destination);
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = pieceValue[board->types[to]] - KNIGHT_VALUE;
			moves->size ++;
		}
	}
	/* Bishop moves */
	pieces = board->bishops[player];
	while (pieces) {
		from = getFirstBitAndClear64(&pieces);
		attack = attackBishop(board,from);
		destination = attack & board->pieces[opponent]; /* Captures */
		while(destination) {
			to = getFirstBitAndClear64(&destination);
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = pieceValue[board->types[to]] - BISHOP_VALUE;
			moves->size ++;
		}
	}

	/* Rook moves */
	pieces = board->rooks[player];
	while (pieces) {
		from = getFirstBitAndClear64(&pieces);
		attack = attackRook(board,from);
		destination = attack & board->pieces[opponent]; /* Captures */
		while(destination) {
			to = getFirstBitAndClear64(&destination);
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = pieceValue[board->types[to]] - ROOK_VALUE;
			moves->size ++;
		}
	}
	
	/* Queen moves */
	pieces = board->queens[player];
	while (pieces) {
		from = getFirstBitAndClear64(&pieces);
		attack = attackQueen(board,from);
		destination = attack & board->pieces[opponent]; /* Captures */
		while(destination) {
			to = getFirstBitAndClear64(&destination);
			moves->move[moves->size] = moveGenerate(from,to);
			moves->value[moves->size] = pieceValue[board->types[to]] - QUEEN_VALUE;
			moves->size ++;
		}
	}

	/* King moves */
	from = indexOfFirstBit64(&board->king[player]);		
	destination = attackKing[from] & board->pieces[opponent];
	while(destination) {
		to = getFirstBitAndClear64(&destination);
		moves->move[moves->size] = moveGenerate(from,to);
		moves->value[moves->size] = pieceValue[board->types[to]] - KING_VALUE;
		moves->size ++;
	}
	moves->next = moves->sorted = 0;
}

void boardSortMoves(Moves* moves){
	guint i,j;
	gint value;
	Move move;
  	for (i = moves->size; i>1; i--)
    	for (j= 0; j< i-1; j++)
			if(moves->value[j+1] > moves->value[j]){
				value = moves->value[j+1];
				moves->value[j+1] = moves->value[j];
				moves->value[j] = value;
			
				move = moves->move[j+1];
				moves->move[j+1] = moves->move[j];
				moves->move[j] = move;
			}
}

void boardGenerateMoves(Board* board, Moves* moves){
	gint history[64][64],i;
	Moves nqmoves;

	boardGenerateQuiescentMoves(board, moves);
	boardGenerateNonQuiescentMoves(board, &nqmoves, history);
	for(i=0; i<nqmoves.size; i++){
		moves->move[i+moves->size] = nqmoves.move[i];
	}

	moves->size += nqmoves.size;
}

void boardGenerateRootMoves(Board* board, RootMoves* moves){
	gint history[64][64],i,index=0;
	Moves gmoves;
	
	boardGenerateQuiescentMoves(board, &gmoves);
	for(i=0; i<gmoves.size; i++){
		if(boardMakeMove(board,gmoves.move[i])){
			moves->move[index].move = gmoves.move[i];
			moves->move[index].value = 0;
			index++;
			boardUnmakeMove(board);
		}
	}	
	boardGenerateNonQuiescentMoves(board, &gmoves, history);
	boardSortMoves(&gmoves);
	for(i=0; i<gmoves.size; i++){
		if(boardMakeMove(board,gmoves.move[i])){
			moves->move[index].move = gmoves.move[i];
			moves->move[index].value = 0;
			index++;
			boardUnmakeMove(board);
		}
	}
	moves->size = index;
}

