/***************************************************************************
 *   Copyright (C) 2006 by Borko Boskovic                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 * \file move.h
 * File contains functionality about moves.
 *
 * Move data structure:<br>
 * 0000000000111111 - index of square from.<br>
 * 0000001111000000 - index of promotion piece (optionally).<br>
 * 1111110000000000 - index of square to.<br>
 */

#ifndef MOVE_H
#define MOVE_H

#include "piece.h"
#include "square.h"

#include <glib.h>

typedef guint16 Move; /**< Type definition of move. */

#define moveGenerate(from,to) (from|(to<<10))		/**< Macro that create and return move. */
#define moveGetFrom(move) (move&63)			/**< Macro that return index of square from. */
#define moveGetTo(move) (move>>10)			/**< Macro that return index of square to. */
#define moveGetPromotion(move) ((move>>6)&15)		/**< Macro that return index of promotion piece. */
#define moveGeneratePromotion(from,to,promotion) (from|(to<<10)|promotion<<6)/**< Macro that set index of promotion piece. */
#define NULL_MOVE 0					/**< Null move. */

/**
 * Function convert move to string.
 * \param move The move to converts into string.
 * \param string The string to write move.
 */
void moveToString(Move move, gchar string[6]);

/**
 * Function convert array of moves into string.
 * \param move The array of moves to convert to string.
 * \param string The string to convert moves.
 */
void movesToString(Move move[], gchar string[]);

/**
 * Prints given move list in long notation to the stdout.
 * \param move move list to print
 */
void movePrintPV(Move move[]);


/**
 * Prints the given move to the stdout in long notation.
 * \param move move to print
 */
void movePrint(Move move);
#endif
