/***************************************************************************
 *   Copyright (C) 2006 by Borko Boskovic                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 * \file piece.h
 * File contains functionality about pieces.
 */

#ifndef PIECE_H
#define PIECE_H

#include "glib.h"

#define WHITE 0 		/**< Presents white player. */
#define BLACK 1			/**< Presents black player. */
#define EMPTY 0			/**< Presents empty piece. */
#define PAWN 2			/**< Presents pawn. */
#define KNIGHT 4		/**< Presents knight. */
#define BISHOP 6		/**< Presents bishop. */
#define ROOK 8			/**< Presents rook. */
#define QUEEN 10		/**< Presents queen. */
#define KING 12			/**< Presents king. */
#define WHITE_PAWN 2		/**< Presents white pawn. */
#define WHITE_KNIGHT 4		/**< Presents white knight. */
#define WHITE_BISHOP 6		/**< Presents white bishop. */
#define WHITE_ROOK 8		/**< Presents white rook. */
#define WHITE_QUEEN 10		/**< Presents white queen. */
#define WHITE_KING 12		/**< Presents white king. */
#define BLACK_PAWN 3		/**< Presents black pawn. */
#define BLACK_KNIGHT 5		/**< Presents black knight. */
#define BLACK_BISHOP 7		/**< Presents black bishop. */
#define BLACK_ROOK 9		/**< Presents black rook. */
#define BLACK_QUEEN 11		/**< Presents black queen. */
#define BLACK_KING 13		/**< Presents black king. */

#define PAWN_VALUE 100		/**< Pawn material value. */
#define KNIGHT_VALUE 300	/**< Knight material value. */
#define BISHOP_VALUE 330	/**< Bishop material value. */
#define ROOK_VALUE 500		/**< Rook material value. */
#define QUEEN_VALUE 900		/**< Queen material value. */
#define KING_VALUE 2000		/**< King material value. */

extern const gint pieceValue[14];	/**< Array of piece material values */
extern const gchar pieceToChar[14];	/**< Array to convert piece index to char */

/**
 * Function convert char to piece index.
 * \param piece The char to convert in piece index.
 * \return Piece index of given char.
 */
gint pieceFromChar(gchar piece);

#endif
