/***************************************************************************
 *   Copyright (C) 2006 by Borko Boskovic                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 * \file square.h
 * Defines square names and other static constants heavily related to square
 * names. First, all square names are represented with the index of bit that
 * is set on bit-board, namely from a1=0 to h8=63. Then file and rank extraction
 * function are defined and individual file and rank masks are defined.
 * For the recreation of a position from it's fen notation, string conversion
 * functions is defined as also the inverse function for giving the answer to
 * "bestmove" query.
 */

#ifndef SQUARE_H
#define SQUARE_H

#include <glib.h>

#define a1 0 /**< Index of the A1 square on the board. */
#define b1 1 /**< Index of the B1 square on the board. */
#define c1 2 /**< Index of the C1 square on the board. */
#define d1 3 /**< Index of the D1 square on the board. */
#define e1 4 /**< Index of the E1 square on the board. */
#define f1 5 /**< Index of the F1 square on the board. */
#define g1 6 /**< Index of the G1 square on the board. */
#define h1 7 /**< Index of the H1 square on the board. */
#define a2 8 /**< Index of the A2 square on the board. */
#define b2 9 /**< Index of the B2 square on the board. */
#define c2 10 /**< Index of the C2 square on the board. */
#define d2 11 /**< Index of the D2 square on the board. */
#define e2 12 /**< Index of the E2 square on the board. */
#define f2 13 /**< Index of the F2 square on the board. */
#define g2 14 /**< Index of the G2 square on the board. */
#define h2 15 /**< Index of the H2 square on the board. */
#define a3 16 /**< Index of the A3 square on the board. */
#define b3 17 /**< Index of the B3 square on the board. */
#define c3 18 /**< Index of the C3 square on the board. */
#define d3 19 /**< Index of the D3 square on the board. */
#define e3 20 /**< Index of the E3 square on the board. */
#define f3 21 /**< Index of the F3 square on the board. */
#define g3 22 /**< Index of the G3 square on the board. */
#define h3 23 /**< Index of the H3 square on the board. */
#define a4 24 /**< Index of the A4 square on the board. */
#define b4 25 /**< Index of the B4 square on the board. */
#define c4 26 /**< Index of the C4 square on the board. */
#define d4 27 /**< Index of the D4 square on the board. */
#define e4 28 /**< Index of the E4 square on the board. */
#define f4 29 /**< Index of the F4 square on the board. */
#define g4 30 /**< Index of the G4 square on the board. */
#define h4 31 /**< Index of the H4 square on the board. */
#define a5 32 /**< Index of the A5 square on the board. */
#define b5 33 /**< Index of the B5 square on the board. */
#define c5 34 /**< Index of the C5 square on the board. */
#define d5 35 /**< Index of the D5 square on the board. */
#define e5 36 /**< Index of the E5 square on the board. */
#define f5 37 /**< Index of the F5 square on the board. */
#define g5 38 /**< Index of the G5 square on the board. */
#define h5 39 /**< Index of the H5 square on the board. */
#define a6 40 /**< Index of the A6 square on the board. */
#define b6 41 /**< Index of the B6 square on the board. */
#define c6 42 /**< Index of the C6 square on the board. */
#define d6 43 /**< Index of the D6 square on the board. */
#define e6 44 /**< Index of the E6 square on the board. */
#define f6 45 /**< Index of the F6 square on the board. */
#define g6 46 /**< Index of the G6 square on the board. */
#define h6 47 /**< Index of the H6 square on the board. */
#define a7 48 /**< Index of the A7 square on the board. */
#define b7 49 /**< Index of the B7 square on the board. */
#define c7 50 /**< Index of the C7 square on the board. */
#define d7 51 /**< Index of the D7 square on the board. */
#define e7 52 /**< Index of the E7 square on the board. */
#define f7 53 /**< Index of the F7 square on the board. */
#define g7 54 /**< Index of the G7 square on the board. */
#define h7 55 /**< Index of the H7 square on the board. */
#define a8 56 /**< Index of the A8 square on the board. */
#define b8 57 /**< Index of the B8 square on the board. */
#define c8 58 /**< Index of the C8 square on the board. */
#define d8 59 /**< Index of the D8 square on the board. */
#define e8 60 /**< Index of the E8 square on the board. */
#define f8 61 /**< Index of the F8 square on the board. */
#define g8 62 /**< Index of the G8 square on the board. */
#define h8 63 /**< Index of the H8 square on the board. */

#define file(s) ((s)&7)  /**< File of square. */
#define rank(s) ((s)>>3) /**< Rank of square. */
#define opposite(s) (s^070) /**< Opposite square */
#define getSquare(f,r) ((r<<3)|f) /**< square index according on file and rank */

#define F1G1 96ull 			/**< Squares the white king passes when short castling. */
#define B1C1D1 14ull 			/**< Squares the white king passes when long castling. */
#define F8G8 6917529027641081856ull /**< Squares the black king passes when
short castling. */
#define B8C8D8 1008806316530991104ull 	/**< Squares the black king passes when
long castling. */
#define G2 (1ull<<g2)
#define G7 (1ull<<g7)
#define B2 (1ull<<b2)
#define B7 (1ull<<b7)
#define H2H3 ((1ull<<h2)|(1ull<<h3))
#define H7H6 ((1ull<<h7)|(1ull<<h6))
#define A2A3 ((1ull<<a2)|(1ull<<a3))
#define A7A6 ((1ull<<a7)|(1ull<<a6))
#define F2G3H2 ((1ull<<f2)|(1ull<<g3)|(1ull<<h2))
#define F7G6H7 ((1ull<<h7)|(1ull<<g6)|(1ull<<h7))
#define A2B3C2 ((1ull<<a2)|(1ull<<b3)|(1ull<<c2))
#define A7B6C7 ((1ull<<a7)|(1ull<<b6)|(1ull<<c7))
#define CENTER ((1ull<<d4)|(1ull<<e4)|(1ull<<d5)|(1ull<<e5))

#define RANK1 255ull			/**< First rank bit-mask. */
#define RANK2 65280ull			/**< Second rank bit-mask. */
#define RANK3 16711680ull		/**< Third rank bit-mask. */
#define RANK4 4278190080ull		/**< Fourth rank bit-mask. */
#define RANK5 1095216660480ull		/**< Fifth rank bit-mask. */
#define RANK6 280375465082880ull	/**< Sixth rank bit-mask. */
#define RANK7 71776119061217280ull	/**< Seventh rank bit-mask. */
#define RANK8 18374686479671623680ull	/**< Eighth rank bit-mask. */

#define FILEA 72340172838076673ull	/**< First rank (A) bit-mask. */
#define FILEB 144680345676153346ull	/**< Second rank (B) bit-mask. */
#define FILEC 289360691352306692ull	/**< Third rank (C) bit-mask. */
#define FILED 578721382704613384ull	/**< Fourth rank (D) bit-mask. */
#define FILEE 1157442765409226768ull	/**< Fifth rank (E) bit-mask. */
#define FILEF 2314885530818453536ull	/**< Sixth rank (F) bit-mask. */
#define FILEG 4629771061636907072ull	/**< Seventh rank (G) bit-mask. */
#define FILEH 9259542123273814144ull	/**< Eighth rank (H) bit-mask. */

#define NOTFILEA 18374403900871474942ull /**< First file (A) bit-mask negation. */
#define NOTFILEH 9187201950435737471ull	 /**< Last file (H) bit-mask negation. */
#define NOTFILEAB 18229723555195321596ull/**< First two (A,B) files bit-mask
negation */
#define NOTFILEGH 4557430888798830399ull /**< Last two (G,H) files bit-mask
negation */

/**
 * Converts index of a square to the long notation string of this square.
 * \param square square index to convert
 * \param string string to write the result into
 */
void squareToString(guint square, gchar string[3]);

/**
 * Calculates the square index from given long notation string of this square.
 * \param string the square in long algebraic notation
 * \return returns index of the square on board (bit-board) 
 */
gint squareFromString(gchar string[3]);

static const guint64 bitFile[8] = {FILEA, FILEB, FILEC, FILED, FILEE, FILEF, FILEG, FILEH};
static const guint64 bitRank[8] = {RANK1, RANK2, RANK3, RANK4, RANK5, RANK6, RANK7, RANK8};
static const guint64 bit7Rank[2] = {RANK7, RANK2};
static const guint rank8[2] = {7,0};
static const guint rank7[2] = {6,1};
gint distance(gint square1, gint square2);
gint cornerDistance(gint square);
#endif
