/***************************************************************************
 *   Copyright (C) 2006 by Borko Boskovic                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdlib.h>

#include "transpositionTable.h"

TranspositionTable* transpositionTableNew(guint mSize){
	TranspositionTable* tTable = malloc(sizeof(TranspositionTable));
	gint dSize = sizeof(TTData);
	tTable->size = tTable->free = (mSize * 1048576)/dSize;
	tTable->data = calloc(tTable->size,dSize);
	return tTable;
}

void transpositionTableReallocate(TranspositionTable* tTable, guint mSize){
	free(tTable->data);
	gint dSize = sizeof(TTData);
	tTable->size = tTable->free = (mSize * 1048576)/dSize;
	tTable->data = calloc(tTable->size,dSize);
}

void transpositionTableFree(TranspositionTable* tTable){
	free(tTable->data);
	free(tTable);
}

guint transpositionTableUsage(TranspositionTable* tTable){
	return (guint)(((tTable->size-tTable->free)/(gfloat)tTable->size)*1000);
}

void transpositionTableWrite(TranspositionTable* tTable, guint64 key, gint depth, 
				guint evaluate, guint flag, Move move, gboolean searchStopCond){
	if(!searchStopCond){
		TTData* tData = &tTable->data[key % tTable->size];
		if(tData->depth <= depth){
			if(tData->depth == 0) tTable->free --;
			tData->depth = depth;
			tData->key = key;
			tData->evaluate = evaluate;
			tData->flag = flag;
			tData->move = move;
		}
	}
}

TTData* transpositionTableFind(TranspositionTable* tTable, guint64 key){
	TTData * tData = &tTable->data[key% tTable->size];
	if(tData->key == key) return tData;
	return NULL;
}

gint transpositionTableDepth(TranspositionTable* tTable, guint64 key){
	return tTable->data[key% tTable->size].depth;
}

void transpositionTableReset(TranspositionTable* tTable){
	gint i;
	for(i=0; i<tTable->size; i++) tTable->data[i].depth = 0;
	tTable->free = tTable->size;
}

void transpositionTableCollectingPV(TranspositionTable* tTable, Board* board, Move pv[MAX_PV+1]){
	TTData* td = transpositionTableFind(tTable, board->key);
	guint index = 0;
	while(td != NULL && index < MAX_PV){
		if(td->move != EMPTY){
			pv[index] = td->move;
			boardMakeMove(board,td->move);
			index++;
		}
		else{
			break;
		}
		td = transpositionTableFind(tTable, board->key);
	}
	pv[index] = EMPTY;
	while(index>0){
		boardUnmakeMove(board);
		index--;
	}
}
